##############################################################################
#
# Copyright (c) 2003 Lars Heber. All Rights Reserved.
# 
##############################################################################

__doc__ = """Unified HTML document module."""
__version__ = '$Revision: 1.0 $'[11:-2]  

from IUnifiedHtmlDocument import IUnifiedHtmlDocument
from Acquisition import Implicit
from Globals import Persistent, InitializeClass, DTMLFile
from AccessControl.Role import RoleManager
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import Item
try:
    from webdav.Lockable import ResourceLockedError
    from webdav.WriteLockInterface import WriteLockInterface
    webdavSupport = 1
except ImportError:
    webdavSupport = 0
from string import find, rfind, lower, join, whitespace

class UnifiedHtmlDocument(Implicit, Persistent, RoleManager, PropertyManager, Item):
    """
    Implements a unified HTML document.
    Basically, an existing HTML document is merged with standard_html_header
    and standard_html_footer in an intelligent way.
    For further documentation please refer to README.txt.
    """
    if webdavSupport:
        __implements__ = (WriteLockInterface, IUnifiedHtmlDocument)
    else:
	__implements__ = IUnifiedHtmlDocument	
    meta_type = "Unified HTML Document"
    content_type = 'text/html'

    _properties = (
	{'id': 'title', 'type': 'string', 'mode': ''},
	{'id': 'Author', 'type': 'string', 'mode': ''},
	)
    security = ClassSecurityInfo()
    manage_options=(
        ({'label': 'Edit', 'action': 'manage_editForm', 'help': ('UnifiedHtmlDocument', 'edit.stx')}, ) +
        ({'label': 'Properties', 'action': 'manage_propertiesForm', 'help': ('UnifiedHtmlDocument', 'properties.stx')}, ) +
	({'label': 'View', 'action': ''}, ) +
	RoleManager.manage_options +
	Item.manage_options
	)
    index_html = None	# avoid acquisition
    security.declareObjectProtected('View')

    
    def __init__(self, id, data):
	"Initialize class instance"
	self.id = id
	self._data = data
	self.getProperties()
    
    def getProperties(self):
	self.title = self.getTitle()
	self.Author = self.getMetaTagValue('DC.Creator')
	if self.Author == "":	self.Author = self.getMetaTagValue('author')


    def findUncommentedString(self, where, what):
	"Find what in where, but what is not within <!-- ... -->"
	# Check if where contains what
	pos = find(where, what)
        if pos == -1:
	    return -1		# Not found at all
	# Check if there is a HTML comment before what
	posCommentStart = rfind(where[:pos], "<!--")
        if posCommentStart == -1:
    	    return pos	# found pos is OK, because what can't be inside a <!-- ... -->
	# Check further if found HTML comment ends before what begins
        posCommentEnd = find(where[posCommentStart:pos], "-->")
        if posCommentEnd > -1:
    	    return pos	# found pos is OK, because found HTML comment ends before what begins
	# Find end of HTML comment
        posCommentEnd = find(where[pos:], "-->")
	if posCommentEnd == -1:
	    return -1	# Strange: No end of HTML comment found...
	# Iterate recursively:
	recursivePos = self.findUncommentedString(where[pos+posCommentEnd+3:], what)        
	if recursivePos == -1:
	    return -1
	return pos+posCommentEnd+3+recursivePos	
	    # Add the previously subtracted part of pos 

    
    def findEndPosOfHtmlTag(self, where, tagStartsWith):
	"""Find position of the end of an HTML tag.
	If you look for the end position of your <body> tag, you search for
	"<body" and you'll get the position of the first character after the
	<body> tag, even if the <body> tag has additional attributes like
	<body bgcolor=#ff0000>.
	"""
	where = lower(where)
	tagStartsWith = lower(tagStartsWith)
	posTag = self.findUncommentedString(where, tagStartsWith)
	if posTag == -1:	return -1
	posTag = posTag + len(tagStartsWith)
	posContent = self.findUncommentedString(where[posTag:], '>')
	if posContent == -1:	return -1
	return posTag + posContent + 1


    security.declarePublic('getTitle')
    def getTitle(self):
	lowerSource = lower(self.document_src())
	dummy = ""
	"Get the title from the first <title> tag"
	posTitleContent = self.findEndPosOfHtmlTag(lowerSource, '<title')
	if posTitleContent == -1:
	    return ""
        posEndTitleTag = self.findUncommentedString(	\
	    lowerSource[posTitleContent:], '</title'	\
	    )
        if posEndTitleTag == -1:
	    return ""
	return self.document_src()[posTitleContent:posTitleContent+posEndTitleTag]


    def getThreePartsAroundHtmlTag(self, src, border):
	# Divide src in three parts (if possible)
	# e. g.: src="blahblah<div class='dummy'>blueblue", border="<div"
	# becomes: ("blahblah", "<div class='dummy'>", "blueblue")
	lowerSrc = lower(src)
	pos1 = self.findUncommentedString(lowerSrc, border)
	if pos1 > -1:
	    pos2 = find(src[pos1:], '>')
	    if pos2 == -1:
		tag = ""
		after = ""
	    else:
		tag = src[pos1:pos1+pos2+1]
		after = src[pos1+pos2+1:]
	    before = src[:pos1]
	else:	# border not found!
	    before = src
	    tag = ""
	    after = ""
	return (before, tag, after)

    
    security.declareProtected('View', '__call__')    
    def __call__(self, REQUEST):
 	"Render the unified HTML document"
	# Retrieve all necessary rendered HTML sources
	headerHtml = self.aq_parent.standard_html_header(self, REQUEST)
	userHtml = self.document_src()
	footerHtml = self.aq_parent.standard_html_footer(self, REQUEST)
	out = ""
	
	# Divide headerHtml into three parts - border: <html>
	(headerHtmlBeforeHtml, headerHtmlHtmlTag,	\
	    headerHtmlAfterHtml) = \
	    self.getThreePartsAroundHtmlTag(headerHtml, '<html')
	if headerHtmlHtmlTag == "":
	    headerHtmlHtmlTag = "<html>"

	# Divide userHtml into three parts - border: <html>
	(userHtmlBeforeHtml, userHtmlHtmlTag,	\
	    userHtmlAfterHtml) = \
	    self.getThreePartsAroundHtmlTag(userHtml, '<html')
	
	##########################################
######
	# Print headerHtmlBeforeHtml, userHtml
BeforeHtml
	out = out + headerHtmlBeforeHtml + userHtmlBeforeHtml

	###############
###########################################
	# Print preferably userHtmlHtmlTag, else footerHtmlHtmlTag
	if userHtmlHtmlTag != "":	out = out + userHtmlHtmlTag
	else:				out = out + footerHtmlHtmlTag


	# Divide headerHtmlAfterHtml into three parts - border: <head>
	(headerHtmlBeforeHead, headerHtmlHeadTag,	\
	    headerHtmlAfterHead) = \
	    self.getThreePartsAroundHtmlTag(headerHtmlAfterHtml, '<head')

	# Divide userHtmlAfterHtml into three parts - border: <head>
	(userHtmlBeforeHead, userHtmlHeadTag,	\
	    userHtmlAfterHead) = \
	    self.getThreePartsAroundHtmlTag(userHtmlAfterHtml, '<head')
	if headerHtmlHeadTag == "":
	    headerHtmlHeadTag = "<head>"
	
	##########################################
######
	# Print headerHtmlBeforeHead, userHtml
BeforeHead
	out = out + headerHtmlBeforeHead + userHtmlBeforeHead

	###############
###########################################
	# Print preferably userHtmlHeadTag, else headerHtmlHeadTag
	if userHtmlHeadTag != "":	out = out + userHtmlHeadTag
	else:				out = out + headerHtmlHeadTag


	# Check if userHtml has valid <title> tag
	lowerUserHtmlAfterHead = lower(userHtmlAfterHead)
	pos1 = self.findUncommentedString(lowerUserHtmlAfterHead, '<title')
	pos2 = self.findEndPosOfHtmlTag(lowerUserHtmlAfterHead, '</title')
	if pos1 > -1 and pos2 > -1 and pos1 < pos2:
	    # <title> tag found in userHtml
	    titleHtml = userHtmlAfterHead[pos1:pos2]
	    # cut <title>...</title> from userHtml:
	    userHtmlAfterHead = userHtmlAfterHead[:pos1] + userHtmlAfterHead[pos2:]
	    
	# extract (if necessary) and cut <title> tag from headerHtml
	lowerHeaderHtmlAfterHead = lower(headerHtmlAfterHead)
	pos1 = self.findUncommentedString(lowerHeaderHtmlAfterHead, '<title')
	pos2 = self.findEndPosOfHtmlTag(lowerHeaderHtmlAfterHead, '</title')
	if pos1 > -1 and pos2 > -1 and pos1 < pos2:
	    if titleHtml == None:	
		# set title, if it doesn't still exist from userHtml
		titleHtml = headerHtmlAfterHead[pos1:pos2]
	    # cut <title>...</title> from headerHtml:
	    headerHtmlAfterHead = headerHtmlAfterHead[:pos1] + headerHtmlAfterHead[pos2:]
	else:
	    titleHtml = ""

	# Divide userHtmlAfterHead into three parts - border: </head>
	(userHtmlBeforeEndHead, userHtmlEndHeadTag, userHtmlAfterEndHead) = \
	    self.getThreePartsAroundHtmlTag(userHtmlAfterHead, '</head')
    
	# Divide headerHtmlAfterHead into three parts - border: </head>
	(headerHtmlBeforeEndHead, headerHtmlEndHeadTag,	\
	    headerHtmlAfterEndHead) = \
	    self.getThreePartsAroundHtmlTag(headerHtmlAfterHead, '</head')
	if headerHtmlEndHeadTag == "":
	    headerHtmlEndHeadTag = "</head>"
    
	##########################################
#######################
	# Print titleHtml, 
headerHtmlBeforeEndHead, userHtmlBeforeEndHead
	out = out + titleHtml + headerHtmlBeforeEndHead + userHtmlBeforeEndHead

	###############
#################################################
	# Print preferably userHtmlEndHeadTag, else headerHtmlEndHeadTag
	if userHtmlEndHeadTag != "":	out = out + userHtmlEndHeadTag
	else:				out = out + headerHtmlEndHeadTag


	# Divide userHtmlAfterEndHead into three parts - border: <body>
	(userHtmlBeforeBody, userHtmlBodyTag, userHtmlAfterBody) = \
	    self.getThreePartsAroundHtmlTag(userHtmlAfterEndHead, '<body')
    
	# Divide headerHtmlAfterEndHead into three parts - border: <body>
	(headerHtmlBeforeBody, headerHtmlBodyTag, headerHtmlAfterBody) = \
	    self.getThreePartsAroundHtmlTag(headerHtmlAfterEndHead, '<body')
	if headerHtmlBodyTag == "":
	    headerHtmlBodyTag = "<body>"
    
	##########################################
#######################
	# Print headerHtmlBeforeBody, userHtmlBeforeBody
	out = out + headerHtmlBeforeBody + userHtmlBeforeBody

	###############
#################################################
	# Print preferably userHtmlBodyTag, else headerHtmlBodyTag
	if userHtmlBodyTag != "":	out = out + userHtmlBodyTag
	else:				out = out + headerHtmlBodyTag

	###########################
	# Print headerHtmlAfterBody
	out = out + headerHtmlAfterBody

	# Divide userHtmlAfterBody into three parts - border: </body>
	(userHtmlBeforeEndBody, userHtmlEndBodyTag, userHtmlAfterEndBody) = \
	    self.getThreePartsAroundHtmlTag(userHtmlAfterBody, '</body')

	# Divide footerHtml into three parts (if possible) - border: </body>
	(footerHtmlBeforeEndBody, footerHtmlEndBodyTag, footerHtmlAfterEndBody) = \
	    self.getThreePartsAroundHtmlTag(footerHtml, '</body')
	if footerHtmlEndBodyTag == "":
	    footerHtmlEndBodyTag = "</body>"

	###########################
###########################
	# Print userHtmlBeforeEndBody, footerHtmlBeforeEndBody
	out = out + userHtmlBeforeEndBody + footerHtmlBeforeEndBody

	###############
#################################################
	# Print preferably userHtmlEndBodyTag, else footerHtmlEndBodyTag
	if userHtmlEndBodyTag != "":	out = out + userHtmlEndBodyTag
	else:				out = out + footerHtmlEndBodyTag


	# Divide userHtmlAfterEndBody into three parts (if possible) - border: </html>
	(userHtmlAfterEndBody, userHtmlEndHtmlTag, userHtmlAfterEndHtml) = \
	    self.getThreePartsAroundHtmlTag(userHtmlAfterEndBody, '</html')

	# Divide footerHtmlAfterEndBody into three parts (if possible) - border: </html>
	(footerHtmlAfterEndBody, footerHtmlEndHtmlTag, footerHtmlAfterEndHtml) = \
	    self.getThreePartsAroundHtmlTag(footerHtmlAfterEndBody, '</html')
	if footerHtmlEndHtmlTag == "":
	    footerHtmlEndHtmlTag = "</html>"
	    
	#####################################################
	# Print userHtmlAfterEndBody
 + footerHtmlAfterEndBody
	out = out + userHtmlAfterEndBody + footerHtmlAfterEndBody

	###############
#################################################
	# Print preferably userHtmlEndHtmlTag, else footerHtmlEndHtmlTag
	if userHtmlEndHtmlTag != "":	out = out + userHtmlEndHtmlTag
	else:				out = out + footerHtmlEndHtmlTag

	###############
######################################
	# Print userHtmlAfterEndHtml + footerHtmlAfterEndHtml
	out = out + userHtmlAfterEndHtml + footerHtmlAfterEndHtml

	return out

	
    security.declareProtected('Access contents information', 'document_src')
    def document_src(self):
	"Return the raw HTML source of the document"
	return self._data


    security.declareProtected('Access contents information', 'getMetaTagValue')
    def getMetaTagValue(self, name, where=None):
	"Return the value of the HTML meta tag with the given name"
	nameString = "name="
	contentString = "content="
	if where is None:	where = self.document_src()
	lowerName = lower(name)
	lowerSrc = lower(where)
	posName = find(lowerSrc, nameString)
	if posName == -1:
	    # Couldn't find nameString at all - nothing to rescue :-(
	    return ""
	posTagEnd = find(lowerSrc[posName:], '>')
	if posTagEnd == -1:
	    # Couldn't find end of Tag
	    return ""
	posNameValue = find(lowerSrc[posName:posName+posTagEnd], lowerName)
	dummyPosNameValue = find(lowerSrc, lowerName)
	# Does the value of name follow immediately or +1 the nameString?
	# And is the found name really equal to the searched name or is the 
	# searched name only a part of the found (e. g.: searched: "author", 
	# but found: "authorOrPublisher" - not what we want!), i. e. does
	# one of whitespace+(">'\") follow the found name?
	if posNameValue < len(nameString) or	\
	    posNameValue > len(nameString)+1 or	\
	    not (where[posName+posNameValue+len(name)] in whitespace+(">'\"")):
	    # Try to find another tag - iterate on shortened text:
	    return self.getMetaTagValue(name,	\
				where[posName+posNameValue+len(name):])
	
	posMeta = rfind(lowerSrc[:posName], '<meta')
	if posMeta == -1 or posMeta > posName:
	    # Didn't find any meta tag - nothing to rescue :-(
	    return ""
	posMetaEnd = posMeta + self.findEndPosOfHtmlTag(lowerSrc[posMeta:], '<meta')
	if posMetaEnd == posMeta-1 or posMetaEnd < posName:
	    # Didn't find the required meta tag - try to find another tag
	    return self.getMetaTagValue(name,	\
				where[posName+posNameValue+len(name):])
	# Finally, we found the desired <meta> tag...
	ourMetaTag = where[posMeta:posMetaEnd]
	posContent = find(ourMetaTag, contentString)
	if posContent == -1:
	    # Didn't find contentString in ourMetaTag - try to find another tag
	    return self.getMetaTagValue(name,	\
				where[posName+posNameValue+len(name):])
	posValueStart=posContent+len(contentString)
	
	if ourMetaTag[posValueStart] in ("'", '"'):	# Value quoted.
	    posValueStart = posValueStart + 1
	    # Find the appropriate closing quotation mark
	    posValueEnd =	\
		find(ourMetaTag[posValueStart:], ourMetaTag[posValueStart-1])
	else:
	    posValueEnd = -1	# Initialize for following loop
	    for c in whitespace + ("\"'>"):
		newPosValueEnd = find(ourMetaTag[posValueStart:], c)
		if newPosValueEnd > -1 and	\
		    (posValueEnd == -1 or newPosValueEnd < posValueEnd):
		    posValueEnd = newPosValueEnd	# new value is better
	    
	if posValueEnd == -1:	# No closing of value found - strange...
	    # Try to find another tag...
	    return self.getMetaTagValue(name,	\
				where[posName+posNameValue+len(name):])
	    
	posValueEnd = posValueStart + posValueEnd
	return ourMetaTag[posValueStart:posValueEnd]


    security.declareProtected('Access contents information', 'get_size')
    def get_size(self):
	"Return the size of the HTML source"
	return len(self._data)
	
    security.declareProtected('Change Unified HTML Documents', 'manage_editForm')
    manage_editForm = DTMLFile('dtml/editForm', globals())

    security.declareProtected('Change Unified HTML Documents', 'manage_edit')
    def manage_edit(self, data, REQUEST=None):
	"Change content"
        if webdavSupport and self.wl_isLocked():
            raise ResourceLockedError, "This %s is locked via WebDAV" % self.meta_type
	self._data = data
	self.getProperties()
        if REQUEST:
            message="Saved changes."
            return self.manage_editForm(self,REQUEST,manage_tabs_message=message)

    def manage_upload(self,file='', REQUEST=None):
        """Replace the contents of the document with the text in file."""
        if webdavSupport and self.wl_isLocked():
            raise ResourceLockedError, "This %s is locked via WebDAV" % self.meta_type

        if type(file) is not type(''): file=file.read()
        self._data = file
	self.getProperties()
        if REQUEST:
            message="File uploaded."
            return self.manage_editForm(self,REQUEST,manage_tabs_message=message)

    security.declareProtected('FTP access', 'manage_FTPget')
    def manage_FTPget(self):
        "Get source for FTP download"
        return self.document_src()


    security.declareProtected('FTP access', 'manage_FTPget')
    def PUT(self, REQUEST, RESPONSE):
	"Handle HTTP/FTP PUT requests."
	self.dav__init(REQUEST, RESPONSE)
	if webdavSupport:
	    self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
	self._data = REQUEST.get('BODY', '')
	self.getProperties()
	RESPONSE.setStatus(204)
	return RESPONSE


manage_addUnifiedHtmlDocumentForm = DTMLFile('dtml/addUnifiedHtmlDocumentForm', globals())

default_html = """
<html>
<head><title>Title of the document</title></head>
<body>
</body>
</html>
"""
from urllib import quote
def manage_addUnifiedHtmlDocument(self, id, file='', REQUEST=None, submit=None):
    """Add a Unified HTML Document with the contents of file. If
    'file' is empty, default document text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file: file=default_html
    instance = UnifiedHtmlDocument(id, file)
    self._setObject(id, instance)
    
    if REQUEST is not None:
	try: u = self.DestinationURL()
	except: u = REQUEST['URL1']
	if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
	REQUEST.RESPONSE.redirect(u+'/manage_workspace')
	return 0
    else:
        ob = getattr(self, id)
        return ob

InitializeClass(UnifiedHtmlDocument)	