##############################################################################
#
# Copyright (c) 2003 Lars Heber. All Rights Reserved.
# 
##############################################################################

"""
UnifiedHtmlDocument: Unified HTML Document

  The classes in this module are available from Unified HTML Documents.
  
"""

class UnifiedHtmlDocument:
    """
    This class implements a special HTML Document which uses 
    *standard_html_header* and *standard_html_footer* in an intelligent way so 
    that features of both (the HTML document and the header / footer), such as 
    CSS, JavaScript etc. are used.
    
    The type of intelligent merging distinguishes this product from the 
    existing HTMLDocument products which are available at 
    "www.zope.org":http://www.zope.org.
    
    The merging of the HTML document with *standard_html_header / ~_footer*  
    makes already available HTML documents which come from external sources 
    (like authors and editors using WYSIWYG tools) fit the needs of an existing 
    Zope-based website.
    
    This means to use the same styles for a uniform style of all pages or 
    always insert the name of the author and the date of last modification at 
    the end of the document, but without actually changing the original HTML 
    document.
    
    Basically, merging is done by mixing up the resulting HTML codes of
    *standard_html_header* and *standard_html_footer* with the original HTML
    source code of the Unified HTML document.
    This is done "on the fly" everytime the Unified HTML Document is called.
	
    Of course, the goal of that merging is the output of **valid** HTML.
    
    The title of the resulting document is taken from the original HTML 
    document, if possible, otherwise, it is taken from *standard_html_header*.
    If this is also not possible, the &lt;title&gt; tag is omitted.
    
    Several properties are generated automagically everytime the document is 
    changed.
    
    For further details, please refer to the README tab of UnifiedHtmlDocument
    in your Products folder or to the Zope online help system.
    """
    
    def __init__(self, id, data):
	"""Initialize a class instance
	
	id -- ID of the new class instance
	
	data -- HTML source code
	"""
    
    def __call__(self, REQUEST):
	"Render the Unified HTML Document"    
    
    def getTitle(self):
	"Get the title from the &lt;title&gt; tag"	
    
    def document_src(self):
	"Return the plain HTML code of the Unified HTML Document"
    
    def getMetaTagValue(self, name):
	"Return the value of the HTML meta tag with the given name"

    def get_size(self):
	"Return the length of the HTML source"